import pytest
from random import randint
from nextcloud_news_filter.config import Config, ConfigurationError


class TestGetConfig:
    def test_address_missing(self, monkeypatch):
        monkeypatch.delenv("NEXTCLOUD_URL", raising=False)
        monkeypatch.setenv("NEXTCLOUD_PASS", "BAR")
        monkeypatch.setenv("NEXTCLOUD_USER", "FOO")
        with pytest.raises(ConfigurationError) as exc_info:
            Config()
        assert "NEXTCLOUD_URL" in exc_info.value.args[0]

    def test_user_is_missing(self, monkeypatch):
        test_url = "https://example.com/"
        monkeypatch.setenv("NEXTCLOUD_URL", test_url)
        monkeypatch.setenv("NEXTCLOUD_PASS", "BAR")
        monkeypatch.delenv("NEXTCLOUD_USER", raising=False)
        with pytest.raises(ConfigurationError) as exc_info:
            Config()
        assert "NEXTCLOUD_USER" in exc_info.value.args[0]

    def test_pass_is_missing(self, monkeypatch):
        test_url = "https://example.com/"
        monkeypatch.setenv("NEXTCLOUD_URL", test_url)
        monkeypatch.setenv("NEXTCLOUD_USER", "BAR")
        monkeypatch.delenv("NEXTCLOUD_PASS", raising=False)
        with pytest.raises(ConfigurationError) as exc_info:
            Config()
        assert "NEXTCLOUD_PASS" in exc_info.value.args[0]

    def test_NEXTCLOUD_URL(self, monkeypatch):
        test_url = "https://example.com/"
        monkeypatch.setenv("NEXTCLOUD_URL", test_url)
        monkeypatch.setenv("NEXTCLOUD_USER", "FOO")
        monkeypatch.setenv("NEXTCLOUD_PASS", "BAR")
        config = Config()
        assert config.nextcloud_url == "https://example.com"

    def test_auth_is_set(self, monkeypatch):
        test_url = "https://example.com/"
        monkeypatch.setenv("NEXTCLOUD_URL", test_url)
        monkeypatch.setenv("NEXTCLOUD_USER", "user")
        monkeypatch.setenv("NEXTCLOUD_PASS", "pass")
        config = Config()
        assert config.auth_header == "Basic dXNlcjpwYXNz"

    def test_default_batchsize_is_set(self, monkeypatch):
        test_url = "https://example.com/"
        monkeypatch.setenv("NEXTCLOUD_URL", test_url)
        monkeypatch.setenv("NEXTCLOUD_USER", "user")
        monkeypatch.setenv("NEXTCLOUD_PASS", "pass")
        config = Config()
        assert config.batch_size == 500

    def test_default_is_overridden(self, monkeypatch):
        test_url = "https://example.com/"
        monkeypatch.setenv("NEXTCLOUD_URL", test_url)
        monkeypatch.setenv("NEXTCLOUD_USER", "user")
        monkeypatch.setenv("NEXTCLOUD_PASS", "pass")
        test_batch_size = randint(0, 300)
        monkeypatch.setenv("BATCH_SIZE", test_batch_size)
        config = Config()
        assert config.batch_size == test_batch_size



import base64
import logging
from os import environ


class ConfigurationError(Exception):
    pass


class Config:
    def __init__(self):
        if environ.get("NEXTCLOUD_URL") is None:
            msg = "NEXTCLOUD_URL is not set in environment"
            logging.log(logging.ERROR, msg)
            raise ConfigurationError(msg)
        self._nextcloud_url = environ["NEXTCLOUD_URL"].rstrip("/")

        if (
            environ.get("NEXTCLOUD_USER") is None
            or environ.get("NEXTCLOUD_PASS") is None
        ):
            msg = "NEXTCLOUD_USER and NEXTCLOUD_PASS need to be set in environment"
            logging.log(
                logging.ERROR,
                msg,
            )
            raise ConfigurationError(msg)
        self._auth_header = f"Basic {self._basic_auth()}"

        self._batch_size = int(environ.get("BATCH_SIZE", 500))

    @property
    def nextcloud_url(self) -> str:
        return self._nextcloud_url

    @property
    def auth_header(self) -> str:
        return self._auth_header

    @property
    def batch_size(self) -> int:
        return self._batch_size

    def _basic_auth(self) -> str:
        return (
            base64.encodebytes(
                f'{environ["NEXTCLOUD_USER"]}:{environ["NEXTCLOUD_PASS"]}'.encode(
                    encoding="UTF-8"
                )
            )
            .decode(encoding="UTF-8")
            .strip()
        )



from datetime import datetime, timedelta
import json
import logging
from pathlib import Path
import re
from typing import Dict, List, Optional, Tuple
import requests

from nextcloud_news_filter import Config


class FilterConfig:
    def __init__(self, filter_json: Dict):
        self._filter = FilterConfig._build_filters(filter_json["filter"])
        if isinstance(feeds_to_skip := filter_json.get("skipFeeds", []), list):
            self._feeds_to_skip = feeds_to_skip

    @property
    def filter(self) -> List[Dict]:
        return self._filter

    @property
    def feeds_to_skip(self) -> List[int]:
        return self._feeds_to_skip

    @classmethod
    def from_file(cls, filter_file: Path) -> Optional["FilterConfig"]:
        try:
            with open(filter_file, "r") as f:
                filter_config = json.loads(f.read())
                return cls(filter_config)
        except FileNotFoundError:
            logging.error(
                f"Can not open file: {filter_file}. Please enter a valid path."
            )

    @staticmethod
    def _build_filters(filters: List[Dict]) -> List[Dict]:
        compiled_filters = []
        for feed_filter in filters:
            one_filter = {
                "name": feed_filter["name"],
                "feedId": feed_filter.get("feedId"),
                "titleRegex": re.compile(feed_filter["titleRegex"], re.IGNORECASE)
                if feed_filter.get("titleRegex")
                else None,
                "bodyRegex": re.compile(feed_filter["bodyRegex"], re.IGNORECASE)
                if feed_filter.get("bodyRegex")
                else None,
                "minPubDate": int(
                    (
                        datetime.now() - timedelta(hours=int(feed_filter["hoursAge"]))
                    ).timestamp()
                )
                if feed_filter.get("hoursAge")
                else None,
            }
            compiled_filters.append(one_filter)
        return compiled_filters


def _apply_filter_to_batch(
    items: Dict, filters_config: FilterConfig
) -> Tuple[List[int], int]:
    unread_item_count = 0
    matched_item_ids = []
    for item in items:
        if item["feedId"] in filters_config.feeds_to_skip:
            logging.debug(f'Skipped because {item["feedId"]}')
            continue
        if item["unread"]:
            unread_item_count = unread_item_count + 1
            for one_filter in filters_config.filter:
                if (
                    (
                        "feedId" not in one_filter
                        or one_filter["feedId"] is None
                        or one_filter["feedId"] == item["feedId"]
                    )
                    and (
                        "titleRegex" not in one_filter
                        or one_filter["titleRegex"] is None
                        or one_filter["titleRegex"].search(item["title"])
                    )
                    and (
                        "bodyRegex" not in one_filter
                        or one_filter["bodyRegex"] is None
                        or one_filter["bodyRegex"].search(item["body"])
                    )
                    and (
                        "minPubDate" not in one_filter
                        or one_filter["minPubDate"] is None
                        or item["pubDate"] < one_filter["minPubDate"]
                    )
                ):
                    logging.log(
                        logging.INFO,
                        f"filter: '{one_filter['name']}' matched item {item['id']} with title {item['title']}",
                    )
                    matched_item_ids.append(item["id"])
    return matched_item_ids, unread_item_count


def filter_items(config: Config, filter_config: FilterConfig) -> Tuple[List[int], int]:
    batch_size = config.batch_size
    offset = 0
    matched_item_ids = []
    unread_item_count = 0
    feed_type = 3  # Feed: 0, Folder: 1, Starred: 2, All: 3
    while True:
        response = requests.get(
            url=f"{config.nextcloud_url}/index.php/apps/news/api/v1-3/items",
            headers=dict(Authorization=config.auth_header),
            json=dict(
                batchSize=batch_size,
                offset=offset,
                type=feed_type,
                id=0,  # 0 = all
                getRead="false",
            ),
        )
        if not response.ok:
            break
        items = response.json()["items"]

        if len(items) == 0:
            break

        matched, count = _apply_filter_to_batch(items, filter_config)
        matched_item_ids = matched_item_ids + matched
        unread_item_count += count
        offset = items[-1]["id"]
    return matched_item_ids, unread_item_count


def mark_as_read(matched_item_ids: List[int], config: Config):
    requests.post(
        url=f"{config.nextcloud_url}/index.php/apps/news/api/v1-3/items/read/multiple",
        headers=dict(Authorization=config.auth_header),
        json=dict(itemIds=matched_item_ids),
    )



try:
    import typer
except ImportError as e:
    e.msg = "You need to install this package with nextcloud_news_filter[cli]"
    raise e

from pathlib import Path
from nextcloud_news_filter import filter_news

from nextcloud_news_filter.filter import FilterConfig


def cli(
    filter_file: Path,
):
    filter_config = FilterConfig.from_file(filter_file=filter_file)
    filter_news(filter_config)


def main():
    typer.run(cli)



import sys
import logging
from os import environ
from nextcloud_news_filter.config import Config

from nextcloud_news_filter.filter import FilterConfig, filter_items, mark_as_read


def handler(*args, **kwargs) -> None:
    logging.basicConfig(
        level=environ.get("LOG_LEVEL", logging.DEBUG),
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        stream=sys.stdout,
    )
    logging.debug(f"Handler called with: {args}")
    filter_config = FilterConfig(args[0]["body"])
    filter_news(filter_config)


def filter_news(filter_config: FilterConfig) -> None:
    logging.debug("starting run")

    config = Config()
    matched_item_ids, unread_item_count = filter_items(config, filter_config)
    if len(matched_item_ids) > 0:
        logging.log(
            logging.INFO,
            f"Marking as read: {len(matched_item_ids)} of {unread_item_count} items.",
        )
        mark_as_read(
            matched_item_ids=matched_item_ids,
            config=config,
        )
    logging.debug("finished run")
